@@ -1,4 +1,4 @@
-/*	Copyright (C) 2004 Bart
+/*	Copyright (C) 2004, 2005, 2006, 2007, 2008 Bart Hakvoort
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,8 +22,8 @@
 #include "../include/DrawingAreaVisualDisk.h"
 #include "../include/Partition.h"
 #include "../include/TreeView_Detail.h"
-#include "../include/Operation.h"
 #include "../include/GParted_Core.h"
+#include "../include/HBoxOperations.h" 
 
 #include <sigc++/class_slot.h>
 #include <gtkmm/paned.h>
@@ -31,15 +31,11 @@
 #include <gtkmm/separatortoolitem.h>
 #include <gtkmm/menubar.h>
 #include <gtkmm/statusbar.h>
-#include <gtkmm/liststore.h>
 #include <gtkmm/combobox.h>
 #include <gtkmm/progressbar.h>
-#include <gtkmm/scrolledwindow.h>
 #include <gtkmm/window.h>
 #include <gtkmm/table.h>
 
-#include <unistd.h> //should be included by gtkmm headers. but decided to include it anyway after getting some bugreports..
-
 namespace GParted
 {
 	
@@ -54,7 +50,6 @@ private:
 	void init_partition_menu() ;
 	Gtk::Menu * create_format_menu() ;
 	void init_device_info() ;
-	void init_operationslist() ;
 	void init_hpaned_main() ;
 
 	void refresh_combo_devices() ;
@@ -63,53 +58,67 @@ private:
 	//Fill txtview_device_info_buffer with some information about the selected device
 	void Fill_Label_Device_Info( bool clear = false );
 
-	void Add_Operation( OperationType operationtype, const Partition & new_partition, int index = -1 ) ;
+	void Add_Operation( Operation * operation, int index = -1 ) ;
 	void Refresh_Visual();
 	bool Quit_Check_Operations();
 	void set_valid_operations() ;
 	
 	//convenience functions
-	void allow_new( bool b )	{ 
-		menu_partition .items()[ 0 ] .set_sensitive( b );
-		toolbar_main .get_nth_item( 0 ) ->set_sensitive( b ); }
-		
-	void allow_delete( bool b )	{ 
-		menu_partition .items()[ 1 ] .set_sensitive( b ); 
-		toolbar_main .get_nth_item( 1 ) ->set_sensitive( b ); }
-		
-	void allow_resize( bool b ) 	{ 
-		menu_partition .items()[ 3 ] .set_sensitive( b ); 
-		toolbar_main .get_nth_item( 3 ) ->set_sensitive( b ); }
-		
-	void allow_copy( bool b )	{ 
-		menu_partition .items()[ 5 ] .set_sensitive( b ); 
-		toolbar_main .get_nth_item( 5 ) ->set_sensitive( b ); }
-		
-	void allow_paste( bool b )	{ 
-		menu_partition .items()[ 6 ] .set_sensitive( b ); 
-		toolbar_main .get_nth_item( 6 ) ->set_sensitive( b ); }
-		
-	void allow_format( bool b )	{ 
-		menu_partition .items()[ 8 ] .set_sensitive( b ); }
-	
-	void allow_toggle_swap_mount_state( bool b )	{ 
-		menu_partition .items()[ 10 ] .set_sensitive( b ); }
+	void toggle_item( bool state, int menu_item, int toolbar_item = -1 )
+        {
+                if ( menu_item >= 0 && menu_item < static_cast<int>( menu_partition .items() .size() ) )
+                        menu_partition .items()[ menu_item ] .set_sensitive( state ) ;
 
-	void allow_manage_flags( bool b ) {
-		menu_partition .items()[ 13 ] .set_sensitive( b ) ; }
-	
-	void allow_info( bool b )	{
-		menu_partition .items()[ 15 ] .set_sensitive( b ); }
-	
-	void allow_undo( bool b )	{ 
-		toolbar_main .get_nth_item( 8 ) ->set_sensitive( b ); 
-		static_cast<Gtk::CheckMenuItem *>( & menubar_main .items()[ 1 ] .get_submenu() ->items()[ 0 ] )
-			->set_sensitive( b ) ; }
+                if ( toolbar_item >= 0 && toolbar_item < toolbar_main .get_n_items() )
+                        toolbar_main .get_nth_item( toolbar_item ) ->set_sensitive( state ) ;
+        }
+
+	void allow_new( bool state )	{ 
+		toggle_item( state, MENU_NEW, TOOLBAR_NEW ) ; }
 		
-	void allow_apply( bool b )	{ 
-		toolbar_main .get_nth_item( 9 ) ->set_sensitive( b ); 
+	void allow_delete( bool state )	{ 
+		toggle_item( state, MENU_DEL, TOOLBAR_DEL ) ; } 
+		
+	void allow_resize( bool state ) 	{ 
+		toggle_item( state, MENU_RESIZE_MOVE, TOOLBAR_RESIZE_MOVE ) ; }
+		
+	void allow_copy( bool state )	{ 
+		toggle_item( state, MENU_COPY, TOOLBAR_COPY ) ; }
+		
+	void allow_paste( bool state )	{ 
+		toggle_item( state, MENU_PASTE, TOOLBAR_PASTE ) ; }
+		
+	void allow_format( bool state )	{ 
+		toggle_item( state, MENU_FORMAT ) ; }
+	
+	void allow_toggle_swap_mount_state( bool state )	{ 
+		toggle_item( state, MENU_TOGGLE_MOUNT_SWAP ) ; }
+
+	void allow_manage_flags( bool state ) {
+		toggle_item( state, MENU_FLAGS ) ; } 
+	
+	void allow_check( bool state ) {
+		toggle_item( state, MENU_CHECK ) ; } 
+	
+	void allow_label_partition( bool state )	{
+		toggle_item( state, MENU_LABEL_PARTITION ) ; } 
+
+	void allow_info( bool state )	{
+		toggle_item( state, MENU_INFO ) ; } 
+
+	void allow_undo_clear_apply( bool state )
+	{
+		toggle_item( state, -1, TOOLBAR_UNDO ) ; 
+		static_cast<Gtk::CheckMenuItem *>( & menubar_main .items()[ 1 ] .get_submenu() ->items()[ 0 ] )
+			->set_sensitive( state ) ; 
+
 		static_cast<Gtk::CheckMenuItem *>( & menubar_main .items()[ 1 ] .get_submenu() ->items()[ 1 ] )
-			->set_sensitive( b ) ; }
+			->set_sensitive( state ) ; 
+
+		toggle_item( state, -1, TOOLBAR_APPLY ) ; 
+		static_cast<Gtk::CheckMenuItem *>( & menubar_main .items()[ 1 ] .get_submenu() ->items()[ 2 ] )
+			->set_sensitive( state ) ; 
+	}
 		
 	//threads..
 	void thread_refresh_devices() ;
@@ -131,6 +140,7 @@ private:
 	void menu_gparted_quit();
 	void menu_view_harddisk_info();
 	void menu_view_operations();
+	void show_help_dialog (const char *link_id);
 	void menu_help_contents();
 	void menu_help_about();
 
@@ -151,6 +161,8 @@ private:
 	void activate_mount_partition( unsigned int index ) ;
 	void activate_disklabel() ;
 	void activate_manage_flags() ;
+	void activate_check() ;
+	void activate_label_partition() ;
 	
 	void activate_undo();
 	void remove_operation( int index = -1, bool remove_all = false ) ;
@@ -165,8 +177,8 @@ private:
 //gui stuff
 	Gtk::HPaned hpaned_main;
 	Gtk::VPaned vpaned_main;
-	Gtk::VBox vbox_main,vbox_info, *vbox;
-	Gtk::HBox hbox_toolbar, hbox_operations, *hbox;
+	Gtk::VBox vbox_main,vbox_info ;
+	Gtk::HBox hbox_toolbar, *hbox;
 	Gtk::Toolbar toolbar_main;
 	Gtk::MenuBar menubar_main;
 	Gtk::ComboBox combo_devices ;
@@ -177,7 +189,6 @@ private:
 	Gtk::ScrolledWindow *scrollwindow;
 	Gtk::Label label_device_info1, label_device_info2, *label;
 	Gtk::Tooltips tooltips ;
-	Gtk::Button *button;
 	Gtk::Table *table ;
 	Gtk::MenuItem *menu_item;
 	Gtk::ProgressBar pulsebar ;
@@ -185,6 +196,7 @@ private:
 	
 	DrawingAreaVisualDisk drawingarea_visualdisk ;
 	TreeView_Detail treeview_detail;
+	HBoxOperations hbox_operations ;
 
 	//device combo
 	Glib::RefPtr<Gtk::ListStore> liststore_devices ;
@@ -204,23 +216,23 @@ private:
 	};
 	treeview_devices_Columns treeview_devices_columns ;
 	
-	//operations list
-	Gtk::TreeView treeview_operations;
-	Glib::RefPtr<Gtk::ListStore> liststore_operations;
-	
-	struct treeview_operations_Columns : public Gtk::TreeModelColumnRecord             
-	{
-		Gtk::TreeModelColumn<Glib::ustring> operation_description;
-		Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> > operation_icon;
-				
-		treeview_operations_Columns() 
-		{ 
-			add( operation_description );
-			add( operation_icon );
-		} 
-	};
-	treeview_operations_Columns treeview_operations_columns;
-	
+	//indices for partitionmenu and toolbar
+        int
+        MENU_NEW, TOOLBAR_NEW,
+        MENU_DEL, TOOLBAR_DEL,
+        MENU_RESIZE_MOVE, TOOLBAR_RESIZE_MOVE,
+        MENU_COPY, TOOLBAR_COPY,
+        MENU_PASTE, TOOLBAR_PASTE,
+        MENU_FORMAT,
+        MENU_TOGGLE_MOUNT_SWAP,
+        MENU_MOUNT,
+        MENU_FLAGS,
+        MENU_CHECK,
+		MENU_LABEL_PARTITION,
+        MENU_INFO,
+        TOOLBAR_UNDO,
+        TOOLBAR_APPLY ;
+
 	//usefull variables which are used by many different functions...
 	int index_extended ; //position of the extended partition (-1 means there isn't one)
 	unsigned short primary_count ;//primary_count checks for max. of 4 pimary partitions
@@ -229,7 +241,6 @@ private:
 	bool OPERATIONSLIST_OPEN ;
 									
 	GParted_Core gparted_core ;
-	GParted::Device *temp_device ;
 	std::vector<Gtk::Label *> device_info ;
 					
 	//stuff for progress overview and pulsebar
