@@ -65,18 +65,16 @@ void DMRaid::load_dmraid_cache()
 {
 	//Load data into dmraid structures
 	Glib::ustring output, error ;
+	dmraid_devices .clear() ;
+
 	if ( dmraid_found )
 	{
 		if ( ! Utils::execute_command( "dmraid -sa -c", output, error, true ) )
 		{
 			Glib::ustring temp = Utils::regexp_label( output, "^(no raid disks).*" ) ;
-			if ( temp == "no raid disks" )
-				dmraid_devices .clear() ;
-			else
+			if ( temp != "no raid disks" )
 				Utils::tokenize( output, dmraid_devices, "\n" ) ;
 		}
-		else
-			dmraid_devices .clear() ;
 	}
 }
 
@@ -104,7 +102,7 @@ bool DMRaid::is_dmraid_device( const Glib::ustring & dev_path )
 	if ( is_dmraid_supported() )
 	{
 		for ( unsigned int k=0; k < dmraid_devices .size(); k++ )
-			if ( dev_path == (DEV_MAP_PATH + dmraid_devices[k]) )
+			if ( dev_path .find( dmraid_devices[k] ) != Glib::ustring::npos )
 				device_found = true ;
 	}
 
@@ -209,11 +207,16 @@ Glib::ustring DMRaid::get_udev_name( const Glib::ustring & dev_path )
 	Glib::ustring udev_name = "" ;
 
 	if ( udevinfo_found )
+	{
 		if ( ! Utils::execute_command( "udevinfo --query=name --name=" + dev_path, output, error, true ) )
 			udev_name = output ;
+	}
 	else if ( udevadm_found )
-		if ( ! Utils::execute_command( "udevadm info --query=name --name=" + DEV_MAP_PATH + dev_path, output, error, true ) )
+	{
+		if ( ! Utils::execute_command( "udevadm info --query=name --name=" + dev_path, output, error, true ) )
 			udev_name = output ;
+	}
+
 	if ( ! udev_name .empty() )
 	{
 		//Extract portion of name after last '/'
