@@ -1,4 +1,4 @@
-/* Copyright (C) 2004, 2005, 2006, 2007, 2008 Bart Hakvoort
+/* Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Bart Hakvoort
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -179,7 +179,7 @@ void Win_GParted::init_menubar()
 
 	//device
 	menu = manage( new Gtk::Menu() ) ;
-	menu ->items() .push_back( Gtk::Menu_Helpers::MenuElem( Glib::ustring( _("_Create Partition Table") ) + " ...",
+	menu ->items() .push_back( Gtk::Menu_Helpers::MenuElem( Glib::ustring( _("_Create Partition Table") ) + "...",
 								sigc::mem_fun(*this, &Win_GParted::activate_disklabel) ) );
 	menubar_main .items() .push_back( Gtk::Menu_Helpers::MenuElem( _("_Device"), *menu ) );
 
@@ -229,8 +229,11 @@ void Win_GParted::init_toolbar()
 	//  FIXME:  Is there a better way to do this, perhaps without the conditional?  At the moment this seems to be the best compromise.
 	bool display_undo = true ;
 	if( str_temp .length() > 14 ) {
-		str_temp .replace( str_temp .find( "/" ), 1, "\n/" ) ;
-		display_undo = false ;
+		size_t index = str_temp .find( "/" ) ;
+		if ( index != Glib::ustring::npos ) {
+			str_temp .replace( index, 1, "\n/" ) ;
+			display_undo = false ;
+		}
 	}
 	toolbutton = Gtk::manage(new Gtk::ToolButton( *image, str_temp ));
 	toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_resize) );
@@ -380,8 +383,16 @@ Gtk::Menu * Win_GParted::create_format_menu()
 {
 	menu = manage( new Gtk::Menu() ) ;
 
-	for ( unsigned int t =0; t < gparted_core .get_filesystems() .size() -1 ; t++ )
+	for ( unsigned int t =0; t < gparted_core .get_filesystems() .size() ; t++ )
 	{
+		//Skip btrfs, luks, lvm2, and unknown because these are not file systems
+		if ( gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_BTRFS ||
+		     gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_LUKS ||
+		     gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_LVM2 ||
+		     gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_UNKNOWN
+		   )
+			continue ;
+
 		hbox = manage( new Gtk::HBox() );
 			
 		//the colored square
@@ -427,7 +438,7 @@ void Win_GParted::init_device_info()
 			0, 1,
 			top, bottom,
 			Gtk::FILL ) ;
-	device_info .push_back( Utils::mk_label( "" ) ) ;
+	device_info .push_back( Utils::mk_label( "", true, Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, false, true ) ) ;
 	table ->attach( * device_info .back(), 1, 2, top++, bottom++, Gtk::FILL ) ;
 	
 	//size
@@ -435,7 +446,7 @@ void Win_GParted::init_device_info()
 			0, 1,
 			top, bottom,
 			Gtk::FILL ) ;
-	device_info .push_back( Utils::mk_label( "" ) ) ;
+	device_info .push_back( Utils::mk_label( "", true, Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, false, true ) ) ;
 	table ->attach( * device_info .back(), 1, 2, top++, bottom++, Gtk::FILL ) ;
 	
 	//path
@@ -446,7 +457,7 @@ void Win_GParted::init_device_info()
 			0, 1,
 			top, bottom,
 			Gtk::FILL ) ;
-	device_info .push_back( Utils::mk_label( "" ) ) ;
+	device_info .push_back( Utils::mk_label( "", true, Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, false, true ) ) ;
 	table ->attach( * device_info .back(), 1, 2, top++, bottom++, Gtk::FILL ) ;
 	
 	vbox_info .pack_start( *table, Gtk::PACK_SHRINK );
@@ -464,7 +475,7 @@ void Win_GParted::init_device_info()
 			0, 1,
 			top, bottom,
 			Gtk::FILL );
-	device_info .push_back( Utils::mk_label( "" ) ) ;
+	device_info .push_back( Utils::mk_label( "", true, Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, false, true ) ) ;
 	table ->attach( * device_info .back(), 1, 2, top++, bottom++, Gtk::FILL ) ;
 	
 	//heads
@@ -472,7 +483,7 @@ void Win_GParted::init_device_info()
 			0, 1,
 			top, bottom,
 			Gtk::FILL ) ;
-	device_info .push_back( Utils::mk_label( "" ) ) ;
+	device_info .push_back( Utils::mk_label( "", true, Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, false, true ) ) ;
 	table ->attach( * device_info .back(), 1, 2, top++, bottom++, Gtk::FILL ) ;
 	
 	//sectors/track
@@ -480,7 +491,7 @@ void Win_GParted::init_device_info()
 			0, 1,
 			top, bottom,
 			Gtk::FILL ) ;
-	device_info .push_back( Utils::mk_label( "" ) ) ;
+	device_info .push_back( Utils::mk_label( "", true, Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, false, true ) ) ;
 	table ->attach( * device_info .back(), 1, 2, top++, bottom++, Gtk::FILL );
 	
 	//cylinders
@@ -488,7 +499,7 @@ void Win_GParted::init_device_info()
 			0, 1,
 			top, bottom,
 			Gtk::FILL ) ;
-	device_info .push_back( Utils::mk_label( "" ) ) ;
+	device_info .push_back( Utils::mk_label( "", true, Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, false, true ) ) ;
 	table ->attach( * device_info .back(), 1, 2, top++, bottom++, Gtk::FILL ) ;
 	
 	//total sectors
@@ -496,7 +507,7 @@ void Win_GParted::init_device_info()
 			0, 1,
 			top, bottom,
 			Gtk::FILL );
-	device_info .push_back( Utils::mk_label( "" ) ) ;
+	device_info .push_back( Utils::mk_label( "", true, Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, false, true ) ) ;
 	table ->attach( * device_info .back(), 1, 2, top++, bottom++, Gtk::FILL ) ;
 	
 	vbox_info .pack_start( *table, Gtk::PACK_SHRINK );
@@ -831,9 +842,50 @@ void Win_GParted::set_valid_operations()
 			else
 				required_size = copied_partition .get_length() ;
 
-			if ( ( required_size + devices[ current_device ] .cylsize ) <= selected_partition .get_length() )
+			//  /* Copy Primary not at start of disk to within Extended partition */
+			//  Adjust when a primary partition is copied and pasted
+			//  into an unallocated space in an extended partition
+			//  of an MSDOS partition table.
+			//  Since the Extended Boot Record requires an additional track,
+			//  this must be considered in the required space for the
+			//  destination (selected) partition.
+			//  NOTE:  This problem does not occur for a primary partition
+			//  at the the start of the disk because the size of the EBR and
+			//  Master Boot Record are the same.
+			//
+			//  /* Copy Primary not at start of disk to Primary at start of disk */
+			//  Also Adjust when a primary partition that does not start at the
+			//  beginning of the disk is copied and pasted
+			//  into an unallocated space at the start of the disk device.
+			//  Since the Master Boot Record requires an additional track,
+			//  this must be considered in the required space for the
+			//  destination (selected) partition.
+			//
+			//  Because the largest unit used in the GUI is one
+			//  cylinder size (round to cylinders), the space
+			//  needed in the destination partition needs to be increased
+			//  by one cylinder size.
+			if (   (/* Copy Primary not at start of disk to within Extended partition */
+				       copied_partition .type == TYPE_PRIMARY
+				    && copied_partition .sector_start > devices[ current_device ] .sectors /* 63 for MSDOS Partition Table */
+				    && devices[ current_device ] .disktype == "msdos" 
+				    && selected_partition .type == TYPE_UNALLOCATED
+				    && selected_partition .inside_extended
+				   )
+				|| ( /* Copy Primary not at start of disk to Primary at start of disk */
+				       copied_partition .type == TYPE_PRIMARY
+				    && copied_partition .sector_start > devices[ current_device ] .sectors /* 63 for MSDOS Partition Table */
+				    && devices[ current_device ] .disktype == "msdos" 
+				    && selected_partition .type == TYPE_UNALLOCATED
+				    && selected_partition .sector_start <= devices[ current_device ] .sectors  /* Beginning of disk device */
+				    && ! selected_partition .inside_extended
+				   )
+			   )
+				required_size += devices[ current_device ] .cylsize ;
+
+			if ( required_size <= selected_partition .get_length() )
 				allow_paste( true ) ;
-		}			
+		}
 		
 		return ;
 	}
@@ -1212,7 +1264,7 @@ void Win_GParted::menu_help_about()
 	dialog .set_logo_icon_name( "gparted" ) ;
 	dialog .set_version( VERSION ) ;
 	dialog .set_comments( _( "GNOME Partition Editor" ) ) ;
-	dialog .set_copyright( "Copyright © 2004-2008 Bart Hakvoort" ) ;
+	dialog .set_copyright( "Copyright © 2004-2009 Bart Hakvoort" ) ;
 
 	//authors
 	strings .push_back( "Bart Hakvoort <gparted@users.sf.net>" ) ;
@@ -1554,8 +1606,8 @@ void Win_GParted::activate_format( GParted::FILESYSTEM new_fs )
 	//check for some limits...
 	fs = gparted_core .get_fs( new_fs ) ;
 	
-	if ( selected_partition .get_length() < fs .MIN ||
-	     fs .MAX && selected_partition .get_length() > fs .MAX ) 
+	if ( ( selected_partition .get_length() < fs .MIN ) ||
+	     ( fs .MAX && selected_partition .get_length() > fs .MAX ) )
 	{
 		Gtk::MessageDialog dialog( *this,
 					   String::ucompose( _("Cannot format this file system to %1."),
@@ -1793,8 +1845,8 @@ void Win_GParted::activate_disklabel()
 {
 	Dialog_Disklabel dialog( devices[ current_device ] .get_path(), gparted_core .get_disklabeltypes() ) ;
 	dialog .set_transient_for( *this );
-		
-	if ( dialog .run() == Gtk::RESPONSE_OK )
+
+	if ( dialog .run() == Gtk::RESPONSE_APPLY )
 	{
 		if ( ! gparted_core .set_disklabel( devices[ current_device ] .get_path(), dialog .Get_Disklabel() ) )
 		{
@@ -1917,7 +1969,11 @@ void Win_GParted::activate_apply()
 				   Gtk::MESSAGE_WARNING,
 				   Gtk::BUTTONS_NONE,
 				   true );
-	dialog .set_secondary_text( _( "It is recommended to backup valuable data before proceeding.") ) ;
+	Glib::ustring temp;
+	temp =  _( "Editing partitions has the potential to cause LOSS of DATA.") ;
+	temp += "\n" ;
+	temp += _( "You are advised to backup your data before proceeding." ) ;
+	dialog .set_secondary_text( temp ) ;
 	dialog .set_title( _( "Apply operations to device" ) );
 	
 	dialog .add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
