@@ -1,4 +1,4 @@
-/* Copyright (C) 2004, 2005, 2006, 2007, 2008 Bart Hakvoort
+/* Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Bart Hakvoort
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ FS fat16::get_filesystem_support()
 	FS fs ;
 	fs .filesystem = GParted::FS_FAT16 ;
 		
-	//find out if we can create fat16 filesystems
+	//find out if we can create fat16 file systems
 	if ( ! Glib::find_program_in_path( "mkdosfs" ) .empty() )
 		fs .create = GParted::FS::EXTERNAL ;
 	
@@ -44,8 +44,8 @@ FS fat16::get_filesystem_support()
 	}
 
 	if ( ! Glib::find_program_in_path( "mlabel" ) .empty() ) {
-		fs .get_label = FS::EXTERNAL ;
-		fs .set_label = FS::EXTERNAL ;
+		fs .read_label = FS::EXTERNAL ;
+		fs .write_label = FS::EXTERNAL ;
 	}
 
 	//resizing of start and endpoint are provided by libparted
@@ -63,7 +63,7 @@ FS fat16::get_filesystem_support()
 
 void fat16::set_used_sectors( Partition & partition ) 
 {
-	exit_status = Utils::execute_command( "dosfsck -a -v " + partition .get_path(), output, error, true ) ;
+	exit_status = Utils::execute_command( "dosfsck -v " + partition .get_path(), output, error, true ) ;
 	if ( exit_status == 0 || exit_status == 1 || exit_status == 256 )
 	{
 		//free clusters
@@ -91,7 +91,7 @@ void fat16::set_used_sectors( Partition & partition )
 	}
 }
 
-void fat16::get_label( Partition & partition )
+void fat16::read_label( Partition & partition )
 {
 	//Create mtools config file
 	char fname[] = "/tmp/gparted-XXXXXXXX" ;
@@ -105,7 +105,7 @@ void fat16::get_label( Partition & partition )
 
 	if ( ! Utils::execute_command( cmd, output, error, true ) )
 	{
-		partition .label = Utils::regexp_label( output, "Volume label is ([^(]*)" );
+		partition .label = Utils::trim( Utils::regexp_label( output, "Volume label is ([^(]*)" ) ) ;
 	}
 	else
 	{
@@ -120,7 +120,7 @@ void fat16::get_label( Partition & partition )
 	err_msg = Utils::delete_mtoolsrc_file( fname );
 }
 
-bool fat16::set_label( const Partition & partition, OperationDetail & operationdetail )
+bool fat16::write_label( const Partition & partition, OperationDetail & operationdetail )
 {
 	//Create mtools config file
 	char fname[] = "/tmp/gparted-XXXXXXXX" ;
@@ -132,7 +132,7 @@ bool fat16::set_label( const Partition & partition, OperationDetail & operationd
 	if( partition .label .empty() )
 		cmd = String::ucompose( "export MTOOLSRC=%1 && mlabel -c %2:", fname, dletter ) ;
 	else
-		cmd = String::ucompose( "export MTOOLSRC=%1 && mlabel %2:\"%3\"", fname, dletter, partition .label ) ;
+		cmd = String::ucompose( "export MTOOLSRC=%1 && mlabel %2:\"%3\"", fname, dletter, Utils::fat_compliant_label( partition .label ) ) ;
 	
 	operationdetail .add_child( OperationDetail( cmd, STATUS_NONE, FONT_BOLD_ITALIC ) ) ;
 	
@@ -152,7 +152,7 @@ bool fat16::set_label( const Partition & partition, OperationDetail & operationd
 
 bool fat16::create( const Partition & new_partition, OperationDetail & operationdetail )
 {
-	return ! execute_command( "mkdosfs -F16 -v -n \"" + new_partition .label + "\" " + new_partition .get_path(), operationdetail ) ;
+	return ! execute_command( "mkdosfs -F16 -v -n \"" + Utils::fat_compliant_label( new_partition .label ) + "\" " + new_partition .get_path(), operationdetail ) ;
 }
 
 bool fat16::resize( const Partition & partition_new, OperationDetail & operationdetail, bool fill_partition )
